import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * A state of <i>the river-crossing problem</i>.
 * <p>
 * A man has a wolf, a sheep and a cabbage, and wants to transport them from 
 * one side of a river to the other using a boat. The boat can only hold the 
 * man and (at most) one item at a time. If left alone with the sheep, the wolf
 * will eat it. The same will happen to the cabbage, in case it is left alone
 * with the sheep. How can the man bring all items safely across the river?
 *
 */ 
public class CrossTheRiverState implements State<CrossTheRiverState>
{

   /**
    * Whether, in this state of the problem, the man is on the other side of 
    * the river or not.
    */
   private boolean man;

   /**    
    * Whether, in this state of the problem, the wolf is on the other side of 
    * the river or not.
    */
   private boolean wolf;

   /**
    * Whether, in this state of the problem, the sheep is on the other side of
    * the river or not.
    */
   private boolean sheep;

   /**
    * Whether, in this state of the problem, the cabbage is on the other side of
    * the river or not.
    */
   private boolean cabbage;

   /**
    * The state, from which this state was reached.
    */
   private CrossTheRiverState previous;

   /**
    * Constructs a new state.
    */
   public CrossTheRiverState()
   {
   }

   /**
    * Constructs a new state based on the given one and with the given previous 
    * state.
    * 
    * @param state the state to copy.
    * @param previous the previous state of the new state.
    */
   private CrossTheRiverState(CrossTheRiverState state, 
      CrossTheRiverState previous)
   {
      this.man = state.man;
      this.wolf = state.wolf;
      this.sheep = state.sheep;
      this.cabbage = state.cabbage;
      this.previous = previous;
   }

   /**
    * Checks whether this state is bad (i.e. it implies that one of the items
    * is about to be eaten).
    *
    * @return {@code true}, if this state is bad, and {@code false}, otherwise.
    */
   private boolean isBad()
   {
      return (sheep == cabbage && cabbage != man) ||
             (wolf == sheep && sheep != man);
   }

   /**
    * Checks whether this state is final, which in this case implies that the man,
    * the wolf, the sheep and the cabbage have all crossed the river in one piece.
    */
   public boolean isFinal()
   {
      return man && wolf && sheep && cabbage;
   }

   /**
    * {@inheritDoc}
    */
   public Collection<CrossTheRiverState> next()
   {
      final List<CrossTheRiverState> n = new ArrayList<CrossTheRiverState>();
      
      // The man must cross the river.
      final CrossTheRiverState s = new CrossTheRiverState(this, this);
      s.man = !s.man;
      if(!s.isBad())
         n.add(s);

      // If the cabbage is on the same side of the river with the man, then 
      // the cabbage can go with him.
      if(cabbage == man)
      {
         final CrossTheRiverState ss = new CrossTheRiverState(s, this);
         ss.cabbage = !ss.cabbage;
         if(!ss.isBad())
            n.add(ss);
      }

      // If the sheep is on the same side of the river with the man, then 
      // the sheep can go with him.
      if(sheep == man)
      {
         final CrossTheRiverState ss = new CrossTheRiverState(s, this);
         ss.sheep = !ss.sheep;
         if(!ss.isBad())
            n.add(ss);
      }

      // If the wolf is on the same side of the river with the man, then   
      // the wolf can go with him.
      if(wolf == man)
      {
         final CrossTheRiverState ss = new CrossTheRiverState(s, this);
         ss.wolf = !ss.wolf;   
         if(!ss.isBad())   
            n.add(ss);   
      }

      return n;
   }

   /**
    * {@inheritDoc}
    */
   public CrossTheRiverState previous()
   {
      return previous;
   }

   /**
    * Returns a string representation of this state, which describes the current
    * position of all persons / animals / items.
    */
   public String toString ()
   {
      final StringBuilder sb = new StringBuilder(" | ");
      
      if(man) sb.append(" man"); else sb.insert(0, "man ");
      if(wolf) sb.append(" wolf"); else sb.insert(0, "wolf ");
      if(sheep) sb.append(" sheep"); else sb.insert(0, "sheep ");
      if(cabbage) sb.append(" cabbage"); else sb.insert(0, "cabbage ");

      return sb.toString();
   }

}
