import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

/**
 * This class represents solvers that attempt to solve problems by making 
 * random moves in their state space.
 * 
 * @param <S> the type of the states of the problems that this random solver 
 *            can solve.
 * 
 */
public class RandomSolver<S extends State<S>> implements Solver<S>
{
    /**
     * A random number generator.
     */
    private final Random random;

    /**
     * Constructs a new random solver.
     */
   public RandomSolver()
   {
      random = new Random();
   }

   /**  
    * Constructs a new random solver with the given seed.
    *
    * @param seed the initial seed.
    */
   public RandomSolver(int seed)
   {
      random = new Random(seed);
   }

   /**
    * {@inheritDoc}
    */
   public S solve(S initial)
   {
      // If the given state is final, just return it.
      if(initial.isFinal())
         return initial;
      
      // Find the states that can be reached from this one.
      final List<S> n = (List<S>) initial.next();

      // If there are no reachable states, just return null.
      if(n.isEmpty())
         return null;

      // Otherwise, pick one of them (randomly) and continue the search from it.
      final S s = n.get(random.nextInt(n.size()));
      return solve(s);
   }

}
