import java.util.Collection;

/**
 * A state that a problem can be in.
 * <p>
 * There are problems that can be modelled as state spaces. A <i>state space</i> 
 * is essentially the set of the states that a problem can be in. There are 
 * <i>initial</i>, <i>final</i> and <i>intermediate</i> states. Each state is 
 * connected to all states that can be reached from it.
 * <p>
 * Solving such problems implies exploring the corresponding state space, 
 * starting from an initial state {@code I}, and trying to reach one of the 
 * final states {@code F}. The solution to the problem might be {@code F} itself 
 * or the path from {@code I} to {@code F}.
 *
 * @param <S> the type of this state (this is similar to the curiously recurring
 *            template pattern in C++)
 *
 */
public interface State<S extends State<S>>
{

   /**
    * Gets the next states of this state, namely the states that can be reached 
    * from this state.
    * 
    * @return a collection that contains the next states of this state.
    */
   public Collection<S> next();

   /**
    * Gets the previous state of this state, namely the state from which we ended
    * up to this one.
    *
    * @return the previous state of this state.
    */
   public S previous();

   /**
    * Determines whether this state is final.
    *
    * @return {@code true}, if this state is final, and {@code false},
    *         otherwise.
    */ 
   public boolean isFinal();

}
